@@ -1,15 +1,10 @@
 ###################################################################################################
 #                              MIT Licence (C) 2022 Cubicpath@Github                              #
 ###################################################################################################
-"""Package initialization module. Import the Client and version info from here."""
-
-__all__ = (
-    'Client',
-)
-
+"""A simple way to get live Halo data straight from Halo Waypoint."""
+# TODO: Add logging functionality
 from ._version import __version__
 from ._version import __version_info__
-from .client import Client
 
 __author__ = 'Cubicpath@Github <cubicpath@pm.me>'
 """Author's information."""
